import 'package:kmp_method/kmp_method.dart';
import 'package:test/test.dart';

void main(){

  group('In English and short source(短い英語の検索元文章に対するテスト)', (){
    //検索元文字列のセット
    var sourceText = 'Hello World';
    var searchText = 'llo';
    var kmp = KmpMethod(sourceText, searchText);
    
    test('searching "llo"(文字列"llo"の探索)', (){

      var matched_point = GetTextMatchedPoint(sourceText, searchText);

      print(matched_point);

      var result = kmp.GetMatchedPoint();
      print(kmp.GetSearchTextLength());
      print(result);

      expect(result, equals(matched_point));
    });

    test('get SourceText length(検索元文字列の長さを取得)', (){
      expect(kmp.GetSourceTextLength(), sourceText.length);
    });

    test('get source text(検索元文字列を取得)',(){
      expect(kmp.GetSourceText(), sourceText);
    });

    test('get search text(検索元文字列を取得)',(){
      expect(kmp.GetSearchText(), searchText);
    });

    test('get SearchText length(検索文字列の長さを取得)',(){
      expect(kmp.GetSearchTextLength(), searchText.length);
    });

    test('set SearchText(新しい検索文字列の設定)',(){
      var newSearchText = 'Hello';

      expect(kmp.GetSearchText(), searchText);
      expect(kmp.GetSearchTextLength(), searchText.length);

      kmp.SetSearchText(newSearchText);

      expect(kmp.GetSearchText(), newSearchText);
      expect(kmp.GetSearchTextLength(), newSearchText.length);
    });

  });

  group('In English and long source(長い英語の検索元文章に対するテスト)', (){
    var sourceText = 'The quick brown fox jumps over the lazy dog';
    var searchText = 'fox';
    var kmp = KmpMethod(sourceText,searchText);

    test('searching "fox"(文字列"fox"の探索)', (){
      var matched_point = GetTextMatchedPoint(sourceText, searchText);
      var result = kmp.GetMatchedPoint();
      print('result:$result\n');

      expect(result, equals(matched_point));
    });

    test('get long source length(検索元文字列の長さを取得)',(){
      expect(kmp.GetSourceTextLength(), sourceText.length);
    });

    test('get source text(検索元文字列の取得)',(){
      expect(kmp.GetSourceText(), sourceText);
    });

    test('get long source text(検索元文字列を取得)', (){
      expect(kmp.GetSourceText(), sourceText);
    });

    test('get search text(検索文字列を取得)', (){
      expect(kmp.GetSearchText(), searchText);
    });

    test('get search text length(検索文字列の長さを取得)',(){
      expect(kmp.GetSearchTextLength(), searchText.length);
    });

    test('change search text(検索文字列の変更)',(){
      var newSearchText = 'brown';

      kmp.SetSearchText(newSearchText);

      expect(kmp.GetSearchText(), newSearchText);
      expect(kmp.GetSearchTextLength(), newSearchText.length);
      expect(kmp.GetMatchedPoint(), GetTextMatchedPoint(sourceText, newSearchText));
    });
  });

  group('In Japanese short source(短い日本語の検索元文章に対するテスト)', (){
    var sourceText = 'こんにちは世界。';
    var searchText = '世界';

    var kmp = KmpMethod(sourceText,searchText);

    test('get source text(検索元文字列を取得)', (){
      expect(kmp.GetSourceText(),sourceText);
    });

    test('get source text length(検索元文字列の長さを取得)',(){
      expect(kmp.GetSourceTextLength(), sourceText.length);
    });

    test('get search text(検索文字列の取得)',(){
      expect(kmp.GetSearchText(), searchText);
    });

    test('get search text length(検索文字列の長さを取得)',(){
      expect(kmp.GetSearchTextLength(), searchText.length);
    });

    test('searching "世界"(文字列"世界"の探索)',(){
      expect(kmp.GetMatchedPoint(), GetTextMatchedPoint(sourceText, searchText));
    });

    test('change search text(検索文字列の変更)',(){
      var newSearchText = 'は';
      kmp.SetSearchText(newSearchText);

      expect(kmp.GetSearchText(), newSearchText);
      expect(kmp.GetSearchTextLength(), newSearchText.length);
      expect(kmp.GetMatchedPoint(), GetTextMatchedPoint(sourceText, newSearchText));
    });
  });
}

List<int> GetTextMatchedPoint(String source,String search){
      var matched_point = <int>[];
      var i = 0;
      while(i<=source.lastIndexOf(search) && i != -1){
        i = source.indexOf(search,i+1);
        if(i != -1){
          matched_point.add(i);
        }
      }
  return matched_point;
}